/***************************************************************************
                          kdbxdoc.cpp  -  description
                             -------------------
    begin                : Sat May 12 19:36:13 BST 2001
    copyright            : (C) 2001 by 
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qwidget.h>
#include <qlist.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <ktempfile.h>

// application specific includes
#include "kdbx.h"
#include "kdbxdoc.h"
#include "kdbxview.h"
#include "timeconv.h"

QList<KdbxView> *KdbxDoc::pViewList = 0L;

KdbxDoc::KdbxDoc(QWidget *parent, const char *name) : QObject(parent, name)
{
  if(!pViewList)
  {
    pViewList = new QList<KdbxView>();
  }

  pViewList->setAutoDelete(true);
	emails.setAutoDelete(true);
	folderTree.setAutoDelete(true);
//  hdbx = NULL;
}

KdbxDoc::~KdbxDoc()
{
}

void KdbxDoc::addView(KdbxView *view)
{
  pViewList->append(view);
  view->connections(this);
}

void KdbxDoc::removeView(KdbxView *view)
{
  pViewList->remove(view);
}
void KdbxDoc::setURL(const KURL &url)
{
  doc_url=url;
}

const KURL& KdbxDoc::URL() const
{
  return doc_url;
}

void KdbxDoc::slotUpdateAllViews(KdbxView *sender)
{
  KdbxView *w;
  if(pViewList)
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
      if(w!=sender)
        w->repaint();
    }
  }

}

void KdbxDoc::slotFolderChanged(const QString &fname) {
	if (openEmails(fname))
		emit signalEmailsLoaded();
}

void KdbxDoc::closeDocument()
{
  deleteContents();
}

bool KdbxDoc::loadFolder(const QString& file) {
	CDBX folders;
	CFolder *folder;
	CItem *item;
	
	if (folders.open((char*)file.latin1()) != 0) {
  	KMessageBox::sorry(0, "Loading of "+file+" file failed. \""+folders.perror()+"\"");
  	qDebug("Load failed [%s]\n", file.latin1());
  	return false;
  }
	qDebug("Folder file opened. Now load of tree begins");
	
	folderTree.clear();
	emails.clear();
	emit signalEmailsLoaded();

	if (folders.type() != DBX_TYPE_FOLDER) {
		folder = new CFolder();
		KURL k(file);
		printf("DEBUG: %s, %s", k.url().latin1(), file.latin1());
		folder->fname(k.filename().latin1());
		folder->name(k.filename().latin1());
		folder->child = NULL;
		folder->next = NULL;
		folderTree.append(folder);
		folders.close();
		return true;
/*		KMessageBox::sorry(0, "File "+file+" is not of type \"FOLDER\". You may have a version of outlook newer than 5.5");
		qDebug("Not a folder");
		return false;*/
	}

	qDebug("Building list");
	while ((item = folders.next(0)) != NULL ) {
		folder = item->toFolder();
		folder->child = NULL;
		folder->next = NULL;
		
		folderTree.append(folder);
	}
	
	folder = folderTree.first();
	
	//when a parent is found for a folder, they will be removed from this list
	QList <CFolder> noParentList(folderTree);
	QList <CFolder> cache;
//	QListIterator <CFolder> iter(nochildList);
	
	CFolder *fol;
	qDebug("Iterating List");
	int x = 0, hit = 0;
	while (folder != NULL) {
		if (!folder->structOK() || folder->id() == 0) {
			printf("Folder has a blank data pointer\n");
			folder = folderTree.next();
			continue;
		}
		
		cache.first();
		while ((fol = cache.current())) {
			if (folder->parentid() == fol->id()) {
//				qDebug("Cache HIT");
  			if (fol->child == NULL) {
  				fol->child = folder;
  			} else {
  				folder->next = fol->child;
  				fol->child = folder;
  			}
				cache.remove();
				cache.prepend(fol);
				hit = 1;
				break;
			}
			cache.next();
		}
		if (hit == 1) {
			hit = 0;
			goto next;
		}
		noParentList.first();
		while ((fol = noParentList.current())) {
//			fol = iter.current();
			if (fol->structOK()) {
  			if (folder->parentid() == fol->id()) {
    			if (fol->child == NULL) {
    				fol->child = folder;
    			} else {
    				folder->next = fol->child;
    				fol->child = folder;
    			}
//    			qDebug("Cache Addition");
    			cache.prepend(fol);
    			if (cache.count() > 100) {
//    				qDebug("Cache Removal");
    				cache.removeLast();
    			}
    				
    			break;
//					noParentList.remove();
//	  			printf("DEBUG: Size = %d\n", noParentList.count());
  			}
  		} else {
				printf("Folder has a blank data pointer\n");
			}			
			noParentList.next();
		}
//		iter.toFirst();
	next:
		folder = folderTree.next();
//		qDebug("Next Item [%d]", x++);
	}
	folders.close();
	qDebug("List finished");
	return true;
}

bool KdbxDoc::openDocument(const KURL& url, const char *format /*=0*/)
{
	QString tmpfile;
	
	if (url.isEmpty()) {
		return false;
	}
	
  if (KIO::NetAccess::download( url, tmpfile )) {
    loadFolder(tmpfile);
    KIO::NetAccess::removeTempFile( tmpfile );
  	doc_url = url;
  	emit signalUpdateViews();
  	format=format;
  	return true;
  }

  return false;
}

bool KdbxDoc::openDirectory(const QString &dir)
{
  QString tmpfile;
	
	if (dir.isEmpty()){
		return false;
	}
	tmpfile = dir+"Folders.dbx";
	
	loadFolder(tmpfile);
	emit signalUpdateViews();
	
	doc_url = KURL(dir);
  return true;
}

bool KdbxDoc::openEmails(const QString &file) {
	//Load an Email dbx file
	KURL f = KURL(doc_url);
	CEmail *email;
	CItem *item;
		
	emails.clear();
	
	dbx.close();
	
	if (file == QString::null) //if null was sent, we only want to clear the current emails
		return true;

	f.setFileName(file);
				
	emit signalStatusBar("Loading File...");
  KIO::NetAccess::download( f, tmpfile );

  if (dbx.open(tmpfile.latin1()) != 0) {
		KMessageBox::sorry(0, "Loading of "+tmpfile+" failed. The case of the filename may be incorrect, or you may not have sufficient permissions");
		qDebug("KdbxDoc::openEmails - Loading of "+tmpfile+" failed");
		emit signalStatusBar("Ready");
		return false;
	}
	
	if (dbx.type() != DBX_TYPE_EMAIL) {
		KMessageBox::sorry(0, "File "+tmpfile+" is not of correct type \"EMAILS\". Please choose another file");
		qDebug("KdbxDoc::openEmails - file "+tmpfile+" is wrong type");
		emit signalStatusBar("Ready");
		return false;
	}
	
	while ((item = dbx.next(0)) != NULL) {
		email = item->toEmail();
	
		emails.append(email);
		kapp->processEvents();
	}
	
  emit signalStatusBar("Ready");
	return true;
}

void KdbxDoc::deleteContents()
{
  emails.clear();
  folderTree.clear();
}

/** Load the body for an email */
void KdbxDoc::loadEmailBody(CEmail* email){
	if (email != NULL)
		dbx.getBody(email);
}

/** Remove the body from an email. To be called when body fetched with loadEmailBody is no longer required */
void KdbxDoc::clearEmailBody(CEmail* email){
	if (email != NULL) {
		email->clearBody();
	}		
}

/** Show/hide the preview pane */
void KdbxDoc::slotShowPreview(int i){
	emit signalShowPreview(i);
}

/** Will write the emails in the QList to the file in url */
void KdbxDoc::exportEmails(QList <CEmail> *emaillist, KURL *url){
	CEmail *email;
	QString tmpfile;
	FILE *out;
	time_t date;
	int current, percent, max;
 	KTempFile *temp;
	
	current = 0;
	percent = 0;

	if (emaillist == NULL ) {
		return;
	}
	if (emaillist->isEmpty()) {
		return;
	}
	
	if (url->isLocalFile()) {
		QString file  = url->directory(false)+url->filename();
		if ((out = fopen(file.latin1(), "w")) == NULL) {
			qDebug("Cannot open target file %s", file.latin1());
			return;
		}
		temp = NULL;
	} else {
		temp = new KTempFile();
  	tmpfile = temp->name();
  	if ((out = temp->fstream()) == NULL) {
  		qDebug("Cannot open the tempfile");
  		return;
  	}
	}
		
	max = emaillist->count();
	email = emaillist->first();
	
	while (email != NULL) {
		dbx.getBody(email);
		date = FileTimeToUnixTime(email->date(), NULL);		
		fprintf(out, "From %s %s", email->sender_address(),ctime(&date)); //ctime adds a CR on end
		if (email->isSeen())
			fprintf(out, "Status: RO\n");
			
		fwrite(email->body(), strlen(email->body()), 1, out);
		fprintf(out, "\n\n");
		email->clearBody();
	
		current++;
		percent = (current*100)/max; //will result in a number 0 to 100 I hope (e.g. 200/4 = 50%)
		email = emaillist->next();
		emit signalUpdateProgress(percent);
		kapp->processEvents(); //maxtime=3secs by default
	}
	
	if (url->isLocalFile()) {
		fclose(out);
	} else {
		temp->close();
  	KIO::NetAccess::upload(tmpfile, *url);
  	temp->unlink();
  }
}

/** Export folder and subfolders to the directory pointed to by url */
void KdbxDoc::exportFolder(CFolder *treeitem, KURL *url, int subFolders, int subdir, int level) {
	CFolder *folder;
	CEmail *email;
	CItem *item;
	KURL f = doc_url;
	QString tmpfile, path;
	CDBX hdbx;
		
	folder = treeitem;
	f.setFileName(folder->fname());
	
	//open dbx file		
  KIO::NetAccess::download( f, tmpfile );
	//want to create the subdir it subdir is set
	if (subdir && treeitem->child != NULL) {
		path = url->directory(false, false)+"."+folder->name()+".directory/";
		KURL url2 = *url;
		url2.setPath(path);
		url2.setFileName(NULL);
		ASSERT(KIO::NetAccess::mkdir(url2));
	}
	
	//if we have children on this node, we need to create the output file (prolly blank)
 	FILE *out;
 	KTempFile *tmp;
 	time_t date;
 	//open output file in ouput URL (maybe open local)
 	if (subdir) //if we are using subdirectories, why not use the nice names
 		url->setFileName(folder->name());
 	else
 		url->setFileName(folder->fname());
 	if (url->isLocalFile()) {
 		QString file = url->directory(false)+url->fileName();
 		if ((out = fopen(file.latin1(), "w")) == NULL) {
 			qDebug("Couldn't open \"%s\" for write", file.latin1());
 			return;
 		}
 		tmp = NULL;
 	} else {
 		tmp = new KTempFile();
 		out = tmp->fstream();
 	}
	
	printf("Exporting %s\n", url->path().latin1());
		
	if (hdbx.open(tmpfile) != 0) {
		qDebug("Couldn't open %s for read", tmpfile.latin1());
	} else if (hdbx.type() != DBX_TYPE_EMAIL) {
		qDebug("%s not of type email", tmpfile.latin1());
	} else {			
		//for each email write it to output
		int x = 0;		
		while ((item = hdbx.next(DBX_FLAG_BODY)) != NULL) {
			email = item->toEmail();
  		date = FileTimeToUnixTime(email->date(), NULL);		
  		fprintf(out, "From %s %s", email->sender_address(),ctime(&date)); //ctime adds a CR on end
  		if (email->isSeen())
  			fprintf(out, "Status: RO\n");
			
  		fwrite(email->body(), strlen(email->body()), 1, out);
  		fprintf(out, "\n\n");
  		delete email;
  		x++;
  		emit signalUpdateProgress((x*100)/hdbx.indexCount());
  		kapp->processEvents(); //max 3secs by default
  	}
	}
 	//close dbx file
 	hdbx.close();
 		
 	//close output file (maybe upload)
 	if (url->isLocalFile()) {
 		fclose(out);
 	} else {
 		tmp->close();
 		KIO::NetAccess::upload(tmp->name(), *url);
 	}
	KIO::NetAccess::removeTempFile(tmpfile);
	
	//process "next" items if we are not on initial level
	if (treeitem->next != NULL && level >0) {
		exportFolder(treeitem->next, url, subFolders, subdir, level);
	}
	
	//call recursive for any children
	if (treeitem->child != NULL && subFolders) {
		if (subdir)
			url->setPath(path);
		exportFolder(treeitem->child, url, subFolders, subdir, level+1);
	}	
}
