/***************************************************************************
                          kdbxdoc.h  -  description
                             -------------------
    begin                : Sat May 12 19:36:13 BST 2001
    copyright            : (C) 2001 by 
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KDBXDOC_H
#define KDBXDOC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

#include <functional>
#include <map>
#include <vector>

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>

// include files for KDE
#include <kurl.h>

#include "libdbx++.h"
#include "dbxtree.h"

using namespace std;

// forward declaration of the Kdbx classes
class KdbxView;
class QListViewItem;

/**	KdbxDoc provides a document object for a document-view model.
  *
  * The KdbxDoc class provides a document object that can be used in conjunction with the classes KdbxApp and KdbxView
  * to create a document-view model for standard KDE applications based on KApplication and KMainWindow. Thereby, the document object
  * is created by the KdbxApp instance and contains the document structure with the according methods for manipulation of the document
  * data by KdbxView objects. Also, KdbxDoc contains the methods for serialization of the document data from and to files.
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 1.2 code generation
  */
class KdbxDoc : public QObject
{
  Q_OBJECT
  public:
    /** Constructor for the fileclass of the application */
    KdbxDoc(QWidget *parent, const char *name=0);
    /** Destructor for the fileclass of the application */
    ~KdbxDoc();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void addView(KdbxView *view);
    /** removes a view from the list of currently connected views */
    void removeView(KdbxView *view);
    /** deletes the document's contents */
    void deleteContents();
    /** closes the acutal document */
    void closeDocument();
    /** loads the document by filename and format and emits the updateViews() signal */
    bool openDocument(const KURL& url, const char *format=0);
    /** loads Folders.dbx from directory by dir */
    bool openDirectory(const QString & dir);
    /** opens a dbx file of type emails and reads them in */
    bool openEmails(const QString & file);
    /** returns the KURL of the document */
    const KURL& URL() const;
    /** sets the URL of the document */
	  void setURL(const KURL& url);
  	/** Load an email with a body */
	  void loadEmailBody(CEmail* email);
  /** Remove the body from an email. To be called when body fetched with loadEmailBody is no longer required */
  void clearEmailBody(CEmail* email);
  /** Will write the emails in the QList to the file in url */
  void exportEmails(QList <CEmail> *emaillist, KURL *url);
	
  public slots:
    /** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
     * As this view normally repaints itself, it is excluded from the paintEvent.
     */
    void slotUpdateAllViews(KdbxView *sender);
 		void slotFolderChanged(const QString &fname);
  /** Show/hide the preview pane */
  void slotShowPreview(int);
 		
  public:	
    /** the list of the views currently connected to the document */
    static QList<KdbxView> *pViewList;	
    //list of currently viewable folders
    QList <CFolder> folderTree;
	
    //list of currently loaded emails
		QList <CEmail> emails;
		
		//handle for the DBX file currently open
//		DBX *hdbx;
		CDBX dbx;
    /** Export folder and subfolders to the directory pointed to by url */
    void exportFolder(CFolder *folder, KURL *url, int subFolders, int subdir, int level);
	signals:
		void signalEmailsLoaded();
	  void signalUpdateViews();
  /** Show/hide preview pane */
  void signalShowPreview(int);
	  /** Connects to KdbxApp to update the status bar */
  	void signalStatusBar(const QString &);
    /** Set the progress bar to adv */
    void signalUpdateProgress(int adv);
		
  private:
		bool loadFolder(const QString &file);
    KURL doc_url;

    QString tmpfile;
};

#endif // KDBXDOC_H
