/***************************************************************************
                          kdbxview.h  -  description
                             -------------------
    begin                : Sat May 12 19:36:13 BST 2001
    copyright            : (C) 2001 by 
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KDBXVIEW_H
#define KDBXVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qwidget.h>
#include <kurl.h>
#include <kconfig.h>
#include <qsplitter.h>

#include "kdbxfolderlistitem.h"

#include "dbxtree.h"

class KdbxDoc;
class KListView;
class KTextBrowser;
class QListViewItem;
class KPopupMenu;

/** The KdbxView class provides the view widget for the KdbxApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As KdbxView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the KdbxApp class to manipulate and display
 * the document structure provided by the KdbxDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class KdbxView : public QWidget
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
    KdbxView(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~KdbxView();

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a KdbxApp instance as a parent
     * widget to get to the window document pointer by calling the KdbxApp::getDocument() method.
     *
     * @see KdbxApp#getDocument
     */
    KdbxDoc *getDocument() const;

		void connections(KdbxDoc *doc);
  /** Exports currently selected emails */
  void exportSelEmails(KURL *);
  /** Find currently selected folder and pass back to document object */
  void exportFolder(KURL *url, int subfold, int subdir);
  /** Saves the properties relevant to the view */
  int saveOptions(KConfig *cfg);
  /** Reload the settings from a config file */
  int loadOptions(KConfig *cfg);
		
  private:
  	void buildTree(CFolder*, KdbxFolderListItem*);  	
  	KListView *folderlist;
  	KListView *emaillist;
  	KTextBrowser *browser;
  	KPopupMenu *exportMenu;
  	QSplitter *split1, *split2;

  signals:
  	void signalFolderChanged(const QString &fname);

  public slots:
  	void slotEmailsLoaded();
    /** Document has loaded some folders */
    void slotFoldersLoaded();
  /** Show/hide preview pane */
  void slotShowPreview(int);
  	
  private slots:
  	void slotItemChanged(QListViewItem *item);
  	void slotEmailClicked(QListViewItem *item);
  	void slotEmailDblClicked(QListViewItem *item);
  signals: // Signals
    /** Allow the view to change the status bar in KdbxApp */
    void signalStatusBar(const QString &);
};

#endif // KDBXVIEW_H
